@@ -60,7 +60,11 @@ void              meta_texture_tower_update_area       (MetaTextureTower *tower,
                                                         int               y,
                                                         int               width,
                                                         int               height);
-CoglTexture      *meta_texture_tower_get_paint_texture (MetaTextureTower *tower);
+CoglTexture      *meta_texture_tower_get_paint_texture (MetaTextureTower    *tower,
+                                                        ClutterPaintContext *paint_context);
+
+void meta_texture_tower_set_snippet (MetaTextureTower *tower,
+                                     CoglSnippet      *snippet);
 
 G_END_DECLS
 
